const util = require("../../utils/util");
const {inArray, ab2hex} = util

const plugin = requirePlugin("sdkPlugin").AiLink;


Page({
  data: {
    showWriteInput: false,
    devices: [],
    connected: false,
    chs: [],
    cmd: '',
    name: '',
    deviceId: null,
    historyList: [],
  },
  onLoad: function () {
    var that=this;
    wx.getSystemInfo({
      success: function(res) {
        if (res.platform === 'devtools') {
          let devices = [{
            deviceId: "02:03:04:05:06:07",
            name: "elink",
            localName: "elink",
            mac: "02:03:04:05:06:07",
            RSSI: -69,
            advertisServiceUUIDs:[ "FFE0","FFC0" ],
            analyzeDataText:"0102003241123413"
          }]
          that.setData({
            devices: devices
          })
        }
      }
    })

    let historyList = []
    historyList = wx.getStorageSync('historyList')
    this.setData({
      historyList,
    })
  },

  toConnectWifi() {
    this.offBLECharacteristicValueChange()
    wx.navigateTo({
      // url: '../wifiList/wifiList',
      url: '../funcList/funcList',
    })
  },

  // 初始化蓝牙模块
  openBluetoothAdapter() {
    wx.openBluetoothAdapter({
      success: (res) => {
        console.log('openBluetoothAdapter success', res)
        this.startBluetoothDevicesDiscovery()
      },
      fail: (res) => {
        if (res.errCode === 10001) {
          wx.showToast({
            title: '请打开蓝牙',
            icon: "none"
          })
          wx.onBluetoothAdapterStateChange(function (res) {
            console.log('onBluetoothAdapterStateChange', res)
            if (res.available) {
              this.startBluetoothDevicesDiscovery()
            }
          })
        }
      }
    })
  },
  // 获取本机蓝牙适配器状态
  getBluetoothAdapterState() {
    wx.getBluetoothAdapterState({
      success: (res) => {
        console.log('getBluetoothAdapterState', res)
        if (res.discovering) {
          this.onBluetoothDeviceFound()
        } else if (res.available) {
          this.startBluetoothDevicesDiscovery()
        }
      }
    })
  },
  // 开始搜寻附近的蓝牙外围设备
  startBluetoothDevicesDiscovery() {
    if (this._discoveryStarted) {
      return
    }
    this._discoveryStarted = true
    wx.startBluetoothDevicesDiscovery({
      allowDuplicatesKey: true,
      interval: 1000,//上报设备的间隔
      services: [
        "FFE0",
      ],
      success: (res) => {
        console.log('startBluetoothDevicesDiscovery success', res)
        this.onBluetoothDeviceFound()
      },
    })
  },
  // 停止搜寻附近的蓝牙外围设备
  stopBluetoothDevicesDiscovery() {
    wx.stopBluetoothDevicesDiscovery()
  },

  // 监听寻找到新设备的事件
  onBluetoothDeviceFound() {
    wx.onBluetoothDeviceFound((res) => {
      res.devices.forEach(device => {
        if (!device.name && !device.localName) {
          return
        }
        const foundDevices = this.data.devices
        const idx = inArray(foundDevices, 'deviceId', device.deviceId)
        const data = {}
        // console.log(device)
        // console.log(ab2hex(device.advertisData))
        device.analyzeDataText = ab2hex(device.advertisData).replace(/,/g, '').toUpperCase()

        let buff = device.advertisData.slice(-6)
        device.mac = new Uint8Array(buff) // 保存广播数据中的mac地址，这是由于iOS不直接返回mac地址
        let tempMac = Array.from(device.mac)
        tempMac.reverse()
        device.macAddr = ab2hex(tempMac, ':').toUpperCase()
        if (idx === -1) {
          data[`devices[${foundDevices.length}]`] = device
        } else {
          data[`devices[${idx}]`] = device
        }
        this.setData(data)
      })
      // 按RSSI值排序
      this.data.devices.sort((a,b)=>{
        return b.RSSI - a.RSSI
       })
    })
  },
  // 连接低功耗蓝牙设备
  createBLEConnection(e) {
    this._connLoading = true
    wx.showLoading({
      title: '连接中',
    })
    setTimeout(() => {
      if (this._connLoading) {
        this._connLoading = false
        wx.hideLoading()
      }
    }, 6000)
    const ds = e.currentTarget.dataset
    const index = ds.index
    // 保存当前连接的设备，注意不能从wxml的dataset中直接返回该对象，因为ArrarBuffer类型的数据无法保留
    this._device = this.data.devices[index]
    console.log(this._device)
    const deviceId = ds.deviceId
    const name = ds.name
    this.mac = ds.mac
    wx.createBLEConnection({
      deviceId,
      success: (res) => {
        this.setData({
          connected: true,
          name,
          deviceId,
        })
        console.log("createBLEConnection:success")
        wx.stopBluetoothDevicesDiscovery()
        this.onBLEConnectionStateChange()
        this.getBLEDeviceServices(deviceId)
      },
      fail: res => {
        this._connLoading = false
        wx.hideLoading()
        wx.showToast({
          title: '连接失败',
          icon: 'none'
        })
      }
    })
    // 连接上设备就可以停止蓝牙搜索，减少功耗。
    this.stopBluetoothDevicesDiscovery()
  },

  onBLEConnectionStateChange() {
    wx.onBLEConnectionStateChange((res) => {
      console.log('wx.onBLEConnectionStateChange() ', res.deviceId, res.connected, res.errorCode, res.errorMsg)
      // 该方法回调中可以用于处理连接意外断开等异常情况
      // console.log(`%c device ${res.deviceId} state has changed, connected: ${res.connected}`, 'color: #F26363')
      if (!res.connected) {
        setTimeout(()=>{
          wx.showToast({
            title: '连接已断开',
            icon: 'none'
          })
        },500)
        wx.navigateBack()
        this.setData({
          connected: false,
          showWriteInput: false,
        })
      }
    })
  },

  // 断开与低功耗蓝牙设备的连接
  closeBLEConnection() {
    wx.closeBLEConnection({
      deviceId: this._deviceId
    })
    this.setData({
      connected: false,
      chs: [],
      showWriteInput: false,
    })
  },

  // 获取蓝牙设备的 serviceId
  getBLEDeviceServices(deviceId) {
    wx.getBLEDeviceServices({
      deviceId,
      success: (res) => {
        for (let i = 0; i < res.services.length; i++) {
          if (res.services[i].isPrimary && res.services[i].uuid.indexOf('FFE0') > -1) {
            this.getBLEDeviceCharacteristics(deviceId, res.services[i].uuid)
            return
          }
        }
      }
    })
  },

  // 获取蓝牙设备某个服务中所有特征值(characteristic)
  getBLEDeviceCharacteristics(deviceId, serviceId) {
    this._deviceId = deviceId
    this._serviceId = serviceId
    this._device.serviceId = serviceId

    wx.getBLEDeviceCharacteristics({
      deviceId,
      serviceId,
      success: (res) => {
        console.log('getBLEDeviceCharacteristics success', res.characteristics)

        // 初始化插件
        plugin.initPlugin(res.characteristics, this._device)

        wx.onBLECharacteristicValueChange((characteristic) => {
          // 解析特征值，返回解密后的数据
          let bleData = plugin.parseBleData(characteristic.value)
          console.log(bleData)
          if (bleData.status == 0) {
            console.log("握手成功")
            this._connLoading = false
            wx.hideLoading()
            wx.showToast({
              title: '连接成功',
            })
            setTimeout(()=>{
              this.toConnectWifi()
            },500)
          } else if (bleData.status == 1) {
            let payload = bleData.data //对应协议中的payload数据，可以自行解析该数据

            // console.log(ab2hex(payload, ' '))
            // console.log(ab2hex(bleData.completeData, ' '))

            const idx = inArray(this.data.chs, 'uuid', characteristic.characteristicId)
            const data = {}
            if (idx === -1) {
              data[`chs[${this.data.chs.length}]`] = {
                uuid: characteristic.characteristicId,
                value: ab2hex(bleData.completeData, ' ')
              }
            } else {
              data[`chs[${idx}]`] = {
                uuid: characteristic.characteristicId,
                value: ab2hex(bleData.completeData, ' ')
              }
            }
            this.setData(data)
          } else if (bleData.status == 2) {
            console.log('A6-----^')
          }
        })
      },
      fail(res) {
        console.error('getBLEDeviceCharacteristics', res)
      }
    })
  },
  writeBLECharacteristicValue(buffer, uuid, deviceId, serviceId) {
    // 向蓝牙设备发送一个二进制流数据
    wx.writeBLECharacteristicValue({
      deviceId,
      serviceId,
      characteristicId: uuid,
      value: buffer,
      success(res) {
        console.log('writeBLECharacteristicValue success', res)
        console.log('下发指令==> ' + ab2hex(buffer))
      }
    })
  },

  offBLECharacteristicValueChange() {
    wx.offBLECharacteristicValueChange((res)=>{
      console.log(res)
    })
  },
  closeBluetoothAdapter() {
    wx.closeBluetoothAdapter()
    this._discoveryStarted = false
  },
});